# Getting Started

## Prerequisites
It will be assumed that the Time Series Anomaly Detection (TSAD) and Icinga
operate on the same host.

Icinga 1.x requires NPCDMOD. In Icinga 2.x the functionality is already
included.

### PNP4Nagios' NPCDMOD
1. Get PNP4Nagios from <http://sourceforge.net/projects/pnp4nagios/>.
2. Build PNP4Nagios (might require RRDtool which should be available via
   package management as `rrdtool`): `./configure` followed by `make all`.
Only the the files `src/npcdmod.o` and `sample-config/pnp/npcd.cfg-sample` are
used by TSAD.
3. Add the paths to the library and the configuration to the Icinga
   configuration (usually `/etc/icinga/icinga.cfg`) and also enable the
processing of performance data for Icinga:

    process_performance_data=1
    broker_module=/path/to/npcdmod.o config_file=/path/to/npcd.cfg
4. Configure NPCD (`npcd.cfg`). Look for the `check_result_path` entry in the
   Icinga configuration to get the path of the spool directory and set up the
`perfdata_spool_dir` and `perfdata_file` entries accordingly, e.g.,

    perfdata_spool_dir = /var/lib/icinga/spool/
    perfdata_file = /var/lib/icinga/perfdata.dump
It might be necessary to comment the `perfdata_file_run_cmd` entry, which is
not used.

### R and Rserve
1. Install R through the package management (`r-base`).
2. Install Rserve through R. Start `R` and `install.packages("Rserve")`.
3. Start Rserve. Inside R use
    library(Rserve)
    Rserve()
to load and start Rserve.
4. Optionally note the path displayed by starting Rserve via R and use this
   command in the future to start the daemon directly outside of R.

### send\_nsca
1. Install the two packages `nsca-ng-client` and `nsca-ng-server`.
2. Set up the `send_nsca` configuration in `/etc/send_nsca.cfg` to match the
   server configuration (`nsca-ng.cfg`) for authorization.
3. Make sure the nsca server as well as Icinga use the same `command_file`. Set
    command_file=/var/lib/icinga/rw/icinga.cmd
in both configurations (`nsca-ng.cfg` and `icinga.cfg`).
4. Additionally enable external commands for Icinga. In the `icinga.cfg` set
    check_external_commands = 1
    command_check_interval = 15s
    log_passive_checks = 1

## Usage
Set the spool directory in the `tsad.properties`.

Start Icinga, nsca-ng and Rserve beforehand, then start TSAD directly from the
jar file.

    java -jar /path/to/tsad.jar

### Use Custom Object Definitions
The default configuration (`reuse_icinga_object_definitions=true`) in the
`tsad.properties` allows TSAD to reuse the original object definitions that
the analysis itself is based on. This results in no additional entries for
Icinga, but the performance data flaps between the original report (which can
also include multiple performance measures itself) and the TSAD report (for
each individual performance measure).

To get a single report for each time series custom object definitions are
required. Set `reuse_icinga_object_definitions=false` and create a copy for
each Icinga object definition with the following naming schema and additional
properties.

### Naming Schema
The new name consists of three parts: the TSAD prefix, defined in
`tsad.properties`, followed by the original service description (an empty
string in case of no service), closed by the corresponding performance data
indicator. The performance data indicator depends on the `check_command` and
is hopefully available through the documentation of the check command.
Otherwise the TSAD log or the Icinga log (`/var/log/icinga.log`) should
contain appropriate debug messages including those details, e.g.,

    Warning:  Passive check result was received for service 'tsad-Current Load-load15' on host 'localhost', but the service could not be found!
For example the service

    service_description   Current Load
using the check command

    check_command         check_load!5.0!4.0!3.0!10.0!6.0!4.0
translates to the following **three** custom objects.

    service_description   tsad-Current Load-load1
    service_description   tsad-Current Load-load5
    service_description   tsad-Current Load-load15


### Properties
To disable the original `check_command` and enable passive checks for the
new object definitions add/set the following two properties.

    active_checks_enabled           0
    passive_checks_enabled          1


### Example Object Definition
The Current Load service, predefined by Icinga (as an example)

    define service{
        use                     generic-service
        host_name               localhost
        service_description     Current Load
        check_command           check_load!5.0!4.0!3.0!10.0!6.0!4.0
    }
results in the following three new TSAD object definitions.

    define service{
        use                     generic-service
        host_name               localhost
        service_description     tsad-Current Load-load1
        active_checks_enabled   0
        passive_checks_enabled  1
        check_command           check_load!5.0!4.0!3.0!10.0!6.0!4.0
    }

    define service{
        use                     generic-service
        host_name               localhost
        service_description     tsad-Current Load-load5
        active_checks_enabled   0
        passive_checks_enabled  1
        check_command           check_load!5.0!4.0!3.0!10.0!6.0!4.0
    }

    define service{
        use                     generic-service
        host_name               localhost
        service_description     tsad-Current Load-load15
        active_checks_enabled   0
        passive_checks_enabled  1
        check_command           check_load!5.0!4.0!3.0!10.0!6.0!4.0
    }


## Troubleshooting
* See `/var/log/icinga.log` for log messages related to Icinga and NPCDMOD.
  This file also contains the `send_nsca` messages received by Icinga and
corresponding problems, e.g., an object definition was not found.
* TSAD requires read and write access to the files in the spools directory.
* NPCDMOD requires read access to its configuration file.

# Build from Source

## Add Rserve Libraries to the (local) Maven Repository
Since the used Rserve version (1.8-0) wasn't available through a public
Maven repository and the [Rserve download section][1] seemed to only
provide the latest version, both libraries were included directly within
this project.

Both Rserve libraries in version 1.8-0 are located in the `local-lib-repo`
folder.  Use the provided script `local-lib-repo/install-local-libs.sh`
to add them to the project local Maven repository (using this exact
folder).  Make sure to run the script from within the subfolder,
otherwise Maven won't be able to find the libraries (or update the
`pom.xml` accordingly).

## Build via Maven
Use `mvn install` to generate a jar file of the TSAD project and install
it into the local (not the project local, like Rserve) Maven repository.

In case the Rserve libraries were installed into the wrong location,
`mvn clean` and/or `mvn install -U` can be used to correct this issue.

# License
The TSAD is released under version 3 of the GNU Affero General Public License.

For Third Party licensing information see the THIRD\_PARTY file.

[1]: https://rforge.net/Rserve/files/
